## @file test_instancing.py
# @brief Tests for I4T Instancing
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##


from   i4t.core.instancing import InstancingToken, InstancingTokenClaims, InstancingWarrant
from   pbose.core.PbWS import PbWS
import pytest
from   tests.fixtures.pbwk import keys_sign
import time
from   datetime import datetime


################################################################################
# PbWS
################################################################################


@pytest.mark.parametrize('key', keys_sign, indirect=True)
def test_instancing_token(key):
  # Instancing Token Claim
  it_claims = InstancingTokenClaims(
    iss='iss',
    sub='sub',
    aud='aud',
    # exp=datetime.utcnow(),
    # nbf=datetime.utcnow(),
    # iat=datetime.utcnow(),
    jti='jti',
    public=b'public',
    private=b'private',
    public_key=key.public_key.protobuf,
    serial='serial',
    sku='sku',
    upc=1,
    manufacturer_name='manufacturer_name',
    manufacturer_location='manufacturer_location',
    manufacturer_lot='manufacturer_lot',
    manufacturer_line='manufacturer_line',
  )
  # Instancing Token
  it = InstancingToken(
    protected=b'protected',
    header=b'header',
    payload=it_claims.to_bytes(),
    signature=b'signature',
  )
  # Instancing Warrant
  iw = InstancingWarrant(
    version=1,
    token=it.protobuf,
    private_key=key.protobuf,
  )
  # sign
  temp = PbWS(payload=b'0123456789')
  temp.sign(iw.private_key)
  # verify
  temp.verify(iw.private_key.public_key)
  # InstancingToken
  it = iw.token
  temp.verify(it.payload.public_key)


@pytest.mark.parametrize('key', keys_sign, indirect=True)
def test_instancing_warrant(key):
  iw = InstancingWarrant.from_key_with_claims(
    key=key,
    iss='iss',
    sub='sub',
    aud='aud',
    # exp=datetime.utcnow(),
    # nbf=datetime.utcnow(),
    # iat=datetime.utcnow(),
    jti='jti',
    public=b'public',
    private=b'private',
    serial='serial',
    sku='sku',
    upc=1,
    manufacturer_name='manufacturer_name',
    manufacturer_location='manufacturer_location',
    manufacturer_lot='manufacturer_lot',
    manufacturer_line='manufacturer_line',
  )
  iw.sign(key)

# @pytest.mark.parametrize('key', keys_sign, indirect=True)
# def test_instancing_warrant_alg(key):
#   iw = InstancingWarrant.generate_for_alg_with_claims(
#     alg=alg,
#     iss='iss',
#     sub='sub',
#     aud='aud',
#     exp=time_now,
#     nbf=time_now,
#     iat=time_now,
#     jti='jti',
#     public=b'public',
#     private=b'private',
#     serial='serial',
#     sku='sku',
#     upc=1,
#     manufacturer_name='manufacturer_name',
#     manufacturer_location='manufacturer_location',
#     manufacturer_lot='manufacturer_lot',
#     manufacturer_line='manufacturer_line',
#   )
#   iw.sign(key)

# @pytest.mark.parametrize('key', keys_sign, indirect=True)
# def test_instancing_warrant_should_fail(key):
#   iw = InstancingWarrant.from_key_with_claims(
#     key=key,
#     public_key=key.public_key, # public key is a bad claim in this instance
#   )
#   assert False



## @file conftest.py
# @brief Configuration for tests
#
# @copyright
# Copyright 2018 PbOSE <https://pbose.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##


pytest_plugins = [
  "tests.fixtures.pbwk",
]



## @file channel.py
# @brief I4T Channel
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##

import logging, os
log = logging.getLogger().getChild(__name__)

from i4t import I4TException

from i4t.client.topic import Topic

import paho.mqtt.client as mqtt

class Channel(object):

  def register_topics(channel):
    pass

  def on_invite(channel):
    pass

  def on_join(channel):
    pass

  def on_members(channel, members):
    pass

  ##############################################################################
  # public
  ##############################################################################

  def __init__(self, applet, channel_name):
    self._topics = {}
    self._topics_registered = {}
    self._members = {}
    self.applet = applet
    self.channel_name = channel_name
    self.register_topics()

  def user(self, var, val=None):
    return self.applet.user(var, val)

  def topic(self, topic_name, topic_cls=None):
    # register a topic
    if topic_cls is not None:
      self._topics.pop(topic_name, None)
      # TODO properly close the topic
      self._topics_registered[topic_name] = topic_cls
    # get the topic instance
    else:
      if topic_name in self._topics:
        return self._topics[topic_name]
      elif topic_name in self._topics_registered:
        topic_cls = self._topics_registered[topic_name]
        self._topics[topic_name] = topic_cls(self, topic_name)
        return self._topics[topic_name]
      else:
        #raise I4TException("Topic: {} has not been registered. Pass topic_cls=Topic".format(topic_name))
        topic_cls = Topic
        self._topics[topic_name] = topic_cls(self, topic_name)
        return self._topics[topic_name]

  def join(self):
    self._join_pending = True
    serial = self.applet.client.iwarrant.token.serial
    # RX updates to session
    self.rx('%s/{}'.format(serial), self._rx_session)
    # TODO send from application instance
    self.tx('%s/{}'.format(serial), serial)
    # RX P2P communications
    self.rx('%p/{}'.format(serial), self._rx_p2p)
    # TX request access to channel
    self.tx('%r', serial)
    return self

  def invite(self, member):
    self.applet.invite(member, self.channel_name)

  def rx(self, topic, callback):
    # rx to a topic
    topic = "{}/{}".format(self.channel_name, topic)
    self.applet.rx(topic, callback)
    return self

  def tx(self, topic, data):
    # tx data to a topic
    topic = "{}/{}".format(self.channel_name, topic)
    self.applet.tx(topic, data)
    return self

  @property
  def applet(self):
    return self._applet

  @applet.setter
  def applet(self, applet):
    self._applet = applet

  @property
  def members(self):
    for member in self._members:
      yield member

  ##############################################################################
  # private
  ##############################################################################

  def _on_invite(self):
    # user callback
    self.on_invite()

  def _rx_session(self, data):
    # data is a session token
    # TODO self.stoken = data
    # recieve content encryption key
    # TODO
    # decrypt content encryption key
    # TODO
    if self._join_pending:
      self._join_pending = False
      # user callback
      self.on_join()
    # update membership table
    # TODO self._members[data] = data
    self.on_members(self.members)

  def _rx_p2p(self, data):
    channel_name = data
    self.applet.channel(channel_name)._on_invite()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: i4t/protobuf/Firmware.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from i4t.protobuf import common_pb2 as i4t_dot_protobuf_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='i4t/protobuf/Firmware.proto',
  package='i4t.protobuf',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1bi4t/protobuf/Firmware.proto\x12\x0ci4t.protobuf\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19i4t/protobuf/common.proto\"\xa7\x03\n\x13\x46irmwareTokenClaims\x12\x0b\n\x03iss\x18\x01 \x01(\t\x12\x0b\n\x03sub\x18\x02 \x01(\t\x12\x0b\n\x03\x61ud\x18\x03 \x01(\t\x12\'\n\x03\x65xp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03nbf\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03iat\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0b\n\x03jti\x18\x07 \x01(\t\x12\x0e\n\x06public\x18\x08 \x01(\x0c\x12\x0f\n\x07private\x18\t \x01(\x0c\x12\x0f\n\x07version\x18\x0b \x01(\t\x12\x0c\n\x04size\x18\x0c \x01(\r\x12\x0e\n\x06\x62inary\x18\r \x01(\x0c\x12\x11\n\tfile_name\x18\x0e \x01(\t\x12\x36\n\tfile_type\x18\x0f \x01(\x0e\x32#.i4t.protobuf.FirmwareFileType.Enum\x12\x11\n\tfile_hash\x18\x10 \x01(\x0c\x12\x33\n\x0e\x66ile_hash_type\x18\x11 \x01(\x0e\x32\x1b.i4t.protobuf.HashType.Enum\"\x92\x01\n\x10\x46irmwareFileType\"~\n\x04\x45num\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03\x42IN\x10\x01\x12\x07\n\x03HEX\x10\x02\x12\x07\n\x03\x45LF\x10\x03\x12\x07\n\x03MAP\x10\x04\x12\x07\n\x03IMG\x10\x05\x12\x07\n\x03TAR\x10\x0b\x12\n\n\x06TAR_BZ\x10\x0c\x12\n\n\x06TAR_GZ\x10\r\x12\n\n\x06TAR_XZ\x10\x0e\x12\x07\n\x03ZIP\x10\x15\x12\x06\n\x02Z7\x10\x16\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,i4t_dot_protobuf_dot_common__pb2.DESCRIPTOR,])



_FIRMWAREFILETYPE_ENUM = _descriptor.EnumDescriptor(
  name='Enum',
  full_name='i4t.protobuf.FirmwareFileType.Enum',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BIN', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HEX', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ELF', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MAP', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IMG', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TAR', index=6, number=11,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TAR_BZ', index=7, number=12,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TAR_GZ', index=8, number=13,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TAR_XZ', index=9, number=14,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ZIP', index=10, number=21,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Z7', index=11, number=22,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=552,
  serialized_end=678,
)
_sym_db.RegisterEnumDescriptor(_FIRMWAREFILETYPE_ENUM)


_FIRMWARETOKENCLAIMS = _descriptor.Descriptor(
  name='FirmwareTokenClaims',
  full_name='i4t.protobuf.FirmwareTokenClaims',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='iss', full_name='i4t.protobuf.FirmwareTokenClaims.iss', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sub', full_name='i4t.protobuf.FirmwareTokenClaims.sub', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='aud', full_name='i4t.protobuf.FirmwareTokenClaims.aud', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exp', full_name='i4t.protobuf.FirmwareTokenClaims.exp', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nbf', full_name='i4t.protobuf.FirmwareTokenClaims.nbf', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='iat', full_name='i4t.protobuf.FirmwareTokenClaims.iat', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='jti', full_name='i4t.protobuf.FirmwareTokenClaims.jti', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public', full_name='i4t.protobuf.FirmwareTokenClaims.public', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='private', full_name='i4t.protobuf.FirmwareTokenClaims.private', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='i4t.protobuf.FirmwareTokenClaims.version', index=9,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='i4t.protobuf.FirmwareTokenClaims.size', index=10,
      number=12, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='binary', full_name='i4t.protobuf.FirmwareTokenClaims.binary', index=11,
      number=13, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_name', full_name='i4t.protobuf.FirmwareTokenClaims.file_name', index=12,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_type', full_name='i4t.protobuf.FirmwareTokenClaims.file_type', index=13,
      number=15, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_hash', full_name='i4t.protobuf.FirmwareTokenClaims.file_hash', index=14,
      number=16, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_hash_type', full_name='i4t.protobuf.FirmwareTokenClaims.file_hash_type', index=15,
      number=17, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=529,
)


_FIRMWAREFILETYPE = _descriptor.Descriptor(
  name='FirmwareFileType',
  full_name='i4t.protobuf.FirmwareFileType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FIRMWAREFILETYPE_ENUM,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=532,
  serialized_end=678,
)

_FIRMWARETOKENCLAIMS.fields_by_name['exp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FIRMWARETOKENCLAIMS.fields_by_name['nbf'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FIRMWARETOKENCLAIMS.fields_by_name['iat'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FIRMWARETOKENCLAIMS.fields_by_name['file_type'].enum_type = _FIRMWAREFILETYPE_ENUM
_FIRMWARETOKENCLAIMS.fields_by_name['file_hash_type'].enum_type = i4t_dot_protobuf_dot_common__pb2._HASHTYPE_ENUM
_FIRMWAREFILETYPE_ENUM.containing_type = _FIRMWAREFILETYPE
DESCRIPTOR.message_types_by_name['FirmwareTokenClaims'] = _FIRMWARETOKENCLAIMS
DESCRIPTOR.message_types_by_name['FirmwareFileType'] = _FIRMWAREFILETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FirmwareTokenClaims = _reflection.GeneratedProtocolMessageType('FirmwareTokenClaims', (_message.Message,), dict(
  DESCRIPTOR = _FIRMWARETOKENCLAIMS,
  __module__ = 'i4t.protobuf.Firmware_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.FirmwareTokenClaims)
  ))
_sym_db.RegisterMessage(FirmwareTokenClaims)

FirmwareFileType = _reflection.GeneratedProtocolMessageType('FirmwareFileType', (_message.Message,), dict(
  DESCRIPTOR = _FIRMWAREFILETYPE,
  __module__ = 'i4t.protobuf.Firmware_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.FirmwareFileType)
  ))
_sym_db.RegisterMessage(FirmwareFileType)


# @@protoc_insertion_point(module_scope)



## @file instancing.py
# @brief I4T Instancing Token
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##


from   google.protobuf.timestamp_pb2 import Timestamp
from   i4t.protobuf import Instancing_pb2
from   pbose.core import pb_object
from   pbose.core.PbWK import PbWK
from   pbose.core.PbWS import PbWS


class InstancingToken(PbWS):


  TOKEN_TYPE = 'x.i4t.it'


  # def init_protobuf_with_properties_after(self, protobuf):
  #   assert protobuf.typ == INSTANCING_TOKEN_TYPE


  @property
  def payload(self):
    return InstancingTokenClaims.from_bytes(self.protobuf.payload)


  @payload.setter
  def payload(self, payload):
    if isinstance(payload, PbWS_pb2.InstancingTokenClaims):
      payload = InstancingTokenClaims(payload)
    if isinstance(payload, InstancingTokenClaims):
      self.protobuf.payload = payload.to_bytes()
    else:
      raise TypeError("Not a valid InstancingToken payload.")


class InstancingTokenClaims(pb_object):


  __protobuf__ = Instancing_pb2.InstancingTokenClaims


  @classmethod
  def init_protobuf_with_properties_before(cls, **properties):
    if 'iat' not in properties:
      iat = Timestamp()
      iat.GetCurrentTime()
      properties['iat'] = iat
    return properties


  @property
  def public_key(self):
    return PbWK(self.protobuf.public_key)


class InstancingWarrant(pb_object):


  __protobuf__ = Instancing_pb2.InstancingWarrant


  @classmethod
  def from_key_with_claims(cls, key, **properties):
    assert 'public_key' not in properties.keys()
    itc = InstancingTokenClaims(
      public_key=key.public_key.protobuf,
      **properties,
    )
    it = InstancingToken(
      payload=itc.to_bytes(),
    )
    return cls(
      token=it.protobuf,
      private_key=key.protobuf,
    )


  @property
  def token(self):
    return InstancingToken(self.protobuf.token)


  @property
  def private_key(self):
    return PbWK(self.protobuf.private_key)


  @property
  def signed(self):
    return self.token.signed


  def sign(self, key):
    self.token.sign(key)


  def verify(self, key):
    self.token.verify(key)



## @file topic.py
# @brief I4T Topic
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##

import logging, os
log = logging.getLogger().getChild(__name__)

from i4t import I4TException

import paho.mqtt.client as mqtt

class Topic(object):

  ##############################################################################
  # public
  ##############################################################################

  def __init__(self, channel, topic_name):
    self.channel = channel
    self.topic_name = topic_name

  def user(self, var, val=None):
    return self.channel.user(var, val)

  def rx(self, callback):
    # rx to a topic
    topic = "{}".format(self.topic_name)
    self.channel.rx(topic, callback)
    return self

  def tx(self, data):
    # tx data to a topic
    topic = "{}".format(self.topic_name)
    self.channel.tx(topic, data)
    return self

  @property
  def channel(self):
    return self._channel

  @channel.setter
  def channel(self, channel):
    self._channel = channel

  ##############################################################################
  # private
  ##############################################################################



## @file config.py
# @brief I4T Config
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##

import logging, os
log = logging.getLogger().getChild(__name__)

from i4t.core.instancing import InstancingToken, InstancingWarrant
from i4t.core.provisioning import ProvisioningToken, ProvisioningWarrant

import yaml
import time

I4T_CONFIG_DIR = os.path.expanduser("~/.i4t")
I4T_CONFIG_FILE = "{}/.config.yml".format(I4T_CONFIG_DIR)
I4T_PROJECT_CONFIG_FILENAME = ".project.config.yml"
I4T_IWARRANT_FILENAME = "iwarrant.b64"
I4T_PWARRANT_FILENAME = "pwarrant.b64"

# https://stackoverflow.com/a/600612
def mkdir_p(path):
  try:
    os.makedirs(path)
  except OSError as exc:  # Python >2.5
    import errno
    if exc.errno == errno.EEXIST and os.path.isdir(path):
      pass
    else:
      raise

class Config(object):

  def __init__(self):
    self.read_config()
    self.read_project_config()

  @classmethod
  def default_config(cls):
    return {
      'active': 'dev',
    }

  def set_active(self, active):
    self.config['active'] = active
    self._save_config()

  def get_active(self):
    return self.config['active']

  def read_config(self):
    mkdir_p(I4T_CONFIG_DIR)
    if not os.path.isfile(I4T_CONFIG_FILE):
      config = Config.default_config()
    else:
      with open(I4T_CONFIG_FILE, 'r') as f:
        try:
          config = yaml.load(f)
        except yaml.YAMLError as e:
          log.error(e)
    temp = Config.default_config()
    temp.update(config)
    self.config = temp
    self.save_config()

  def save_config(self):
    with open(I4T_CONFIG_FILE, 'w') as f:
      yaml.dump(self.config, f, default_flow_style=False)

  ##############################################################################
  # Project Config
  ##############################################################################

  @classmethod
  def default_project_config(cls):
    return {
      'mode': 'DEVELOPMENT',
    }

  def _project_config_dir(self):
    _dir = "{}/{}".format(I4T_CONFIG_DIR, self.get_active())
    mkdir_p(_dir)
    return _dir

  def _project_config_file(self):
    return "{}/{}".format(self._project_config_dir(), I4T_PROJECT_CONFIG_FILENAME)

  def read_project_config(self):
    #if not os.path.isfile(self._project_config_file()):
    config = Config.default_project_config()
    with open(self._project_config_file(), 'r+') as f:
      try:
        config = yaml.load(f)
      except yaml.YAMLError as e:
        log.error(e)
    temp = Config.default_project_config()
    temp.update(config)
    self.project_config = temp
    self.save_project_config()

  def save_project_config(self):
    with open(self._project_config_file(), 'w+') as f:
      yaml.dump(self.project_config, f, default_flow_style=False)

  ##############################################################################
  # Project Config - Application
  ##############################################################################

  def _project_config_application_dir(self, application):
    _dir = "{}/applications/{}".format(self._project_config_dir(), application)
    mkdir_p(_dir)
    return _dir

  def _project_config_application_iwarrant(self, application):
    return "{}/{}".format(self._project_config_application_dir(application), I4T_IWARRANT_FILENAME)

  def _project_config_application_pwarrant(self, application):
    return "{}/{}".format(self._project_config_application_dir(application), I4T_PWARRANT_FILENAME)

  def load_project_application(self, application):
    with open(self._project_config_application_iwarrant(application), 'w+') as f:
      iwarrant = InstancingWarrant(f.read())
    with open(self._project_config_application_pwarrant(application), 'w+') as f:
      pwarrant = ProvisioningWarrant(f.read())
    return iwarrant, pwarrant

  def save_project_application(self, application, iwarrant, pwarrant):
    with open(self._project_config_application_iwarrant(application), 'w+') as f:
      f.write(iwarrant.base64)
    with open(self._project_config_application_pwarrant(application), 'w+') as f:
      f.write(pwarrant.base64)

  ##############################################################################
  # Project Config - Client
  ##############################################################################

  def _project_config_client_dir(self, client):
    _dir = "{}/clients/{}".format(self._project_config_dir(), client)
    mkdir_p(_dir)
    return _dir

  def _project_config_client_iwarrant(self, client):
    return "{}/{}".format(self._project_config_client_dir(client), I4T_IWARRANT_FILENAME)

  def _project_config_client_pwarrant(self, client):
    return "{}/{}".format(self._project_config_client_dir(client), I4T_PWARRANT_FILENAME)

  def load_project_client(self, client):
    with open(self._project_config_client_iwarrant(client), 'w+') as f:
      iwarrant = InstancingWarrant(f.read())
    with open(self._project_config_client_pwarrant(client), 'w+') as f:
      pwarrant = ProvisioningWarrant(f.read())
    return iwarrant, pwarrant

  def save_project_client(self, client, iwarrant, pwarrant):
    with open(self._project_config_client_iwarrant(client), 'w+') as f:
      f.write(iwarrant.base64)
    with open(self._project_config_client_pwarrant(client), 'w+') as f:
      f.write(pwarrant.base64)



## @file client.py
# @brief I4T Client
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##

import logging, os
log = logging.getLogger().getChild(__name__)

from i4t import I4TException

from i4t.core.instancing import InstancingToken, InstancingWarrant
from i4t.core.provisioning import ProvisioningToken, ProvisioningWarrant

import paho.mqtt.client as mqtt

class Client(object):

  def on_init(self):
    pass

  def register_applets(self):
    pass

  ##############################################################################
  # public
  ##############################################################################

  def __init__(self, iwarrant):
    self._ptokens = {}
    self._applets = {}
    self._applets_registered = {}
    self._user = {}
    self.iwarrant = iwarrant
    self.on_init()
    self.register_applets()

  def user(self, var, val=None):
    if val is None:
      # get user var
      return self._user[var]
    else:
      # set user var
      self._user[var] = val
      return val

  def applet(self, applet_name, applet_cls=None):
    # register an applet
    if applet_cls is not None:
      self._applets.pop(applet_name, None)
      # TODO properly close the applet
      self._applets_registered[applet_name] = applet_cls
    # get the applet instance
    else:
      if applet_name in self._applets:
        return self._applets[applet_name]
      elif applet_name in self._applets_registered:
        if applet_name in self._ptokens:
          applet_cls = self._applets_registered[applet_name]
          self._applets[applet_name] = applet_cls(self, self._ptokens[applet_name])
          return self._applets[applet_name]
        else:
          raise I4TException("Applet: {} has not been provisioned. Check ProvisioningWarrant.".format(applet_name))
      else:
        raise I4TException("Applet: {} has not been registered. Pass applet_cls=Applet".format(applet_name))

  def provision(self, pwarrant=None):
    if pwarrant is None:
      # TODO: fetch from provisioning server
      pass
    else:
      # set as the provisioning warrant
      self.pwarrant = pwarrant
    return self

  @property
  def iwarrant(self):
    return self._iwarrant

  @iwarrant.setter
  def iwarrant(self, iwarrant):
    self._iwarrant = InstancingWarrant(iwarrant)

  @property
  def pwarrant(self):
    return self._pwarrant

  @pwarrant.setter
  def pwarrant(self, pwarrant):
    self._ptokens = {}
    # TODO properly close each applet
    self._applets = {}
    self._pwarrant = ProvisioningWarrant(pwarrant)
    for token in self._pwarrant.tokens:
      self._ptokens[token.application_name] = token

  ##############################################################################
  # private
  ##############################################################################



## @file instancing.py
# @brief I4T Instancing Token
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##


import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes

from i4t.protobuf import Firmware_pb2
from i4t.protobuf.common_pb2 import HashType
from i4t.protobuf.Firmware_pb2 import FirmwareFileType

from pbose.core import pb_object
from pbose.core.PbWT import PbWT


class FirmwareToken(PbWT):


  TOKEN_TYPE = 'x.i4t.ft'


  @property
  def payload(self):
    return FirmwareTokenClaims.from_bytes(self.protobuf.payload)


  @payload.setter
  def payload(self, payload):
    if isinstance(payload, Firmware_pb2.FirmwareTokenClaims):
      payload = FirmwareTokenClaims(payload)
    if isinstance(payload, FirmwareTokenClaims):
      self.protobuf.payload = payload.to_bytes()
    else:
      raise TypeError("Not a valid FirmwareToken payload.")


class FirmwareTokenClaims(pb_object):


  __protobuf__ = Firmware_pb2.FirmwareTokenClaims


  __file_type_map__ = { # TODO
    FirmwareFileType.TAR_GZ: "tar.gz",
  }


  @classmethod
  def from_file(cls, version, file_path, inline_binary=False):
    if not os.path.isfile(file_path):
      raise ValueError("{} is not a file".format(file_path))
    with open(file_path, 'rb') as fp:
      if inline_binary:
        raise NotImplementedError()
      else:
        digest = hashes.Hash(hashes.SHA256(), backend=default_backend()) # TODO
        chunksize = 1024
        chunk = fp.read(chunksize)
        while chunk:
          digest.update(chunk)
          chunk = fp.read(chunksize)
        file_hash = digest.finalize()
    properties = dict(
      version=version,
      size=os.path.getsize(file_path),
      file_name=os.path.basename(file_path),
      file_type=FirmwareFileType.TAR_GZ, # TODO
      file_hash=file_hash,
      file_hash_type=HashType.SHA256, # TODO
    )
    print(properties)
    return cls(**properties)



## @file pbwk.py
# @brief Fixtures for PbWK
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##


import json
from   pbose.core.PbWA import PbWKAlg
from   pbose.core.PbWK import PbWK, PbWKSet, PbWKType, PbWKUse, PbWKKeyOps
import pytest
import six


################################################################################
# Algorithm Keys
################################################################################


def properties_for_alg(alg):
  alg_str = six.viewkeys(PbWKAlg.Enum)[alg]
  return dict(
    alg=alg,
    kid='test_' + alg_str,
    x5u='https://test_' + alg_str,
    x5c=[b'test'],
    x5t=b'test',
    x5t_s256=b'test',
  )


keys = [
  properties_for_alg(alg)
  for alg in PbWK.__alg_map__
]


keys_ec = [
  properties_for_alg(alg)
  for alg, alg_props in six.iteritems(PbWK.__alg_map__) if PbWKType.EC == alg_props['kty']
]


keys_rsa = [
  properties_for_alg(alg)
  for alg, alg_props in six.iteritems(PbWK.__alg_map__) if PbWKType.RSA == alg_props['kty']
]


keys_oct = [
  properties_for_alg(alg)
  for alg, alg_props in six.iteritems(PbWK.__alg_map__) if PbWKType.OCT == alg_props['kty']
]


keys_sign = [
  properties_for_alg(alg)
  for alg, alg_props in six.iteritems(PbWK.__alg_map__) if PbWKKeyOps.SIGN in alg_props['key_ops']
]


keys_encrypt = [
  properties_for_alg(alg)
  for alg, alg_props in six.iteritems(PbWK.__alg_map__) if PbWKKeyOps.ENCRYPT in alg_props['key_ops']
]


keys_wrapkey = [
  properties_for_alg(alg)
  for alg, alg_props in six.iteritems(PbWK.__alg_map__) if PbWKKeyOps.WRAPKEY in alg_props['key_ops']
]


@pytest.fixture
def key(request):
  properties = request.param
  return PbWK.generate_for_alg(**properties)


@pytest.fixture
def keyset():
  keyset = PbWKSet()
  for key_properties in keys:
    keyset.append(PbWK.generate_for_alg(**key_properties))
  return keyset



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: i4t/protobuf/Instancing.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from pbose.protobuf import PbWK_pb2 as pbose_dot_protobuf_dot_PbWK__pb2
from pbose.protobuf import PbWS_pb2 as pbose_dot_protobuf_dot_PbWS__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='i4t/protobuf/Instancing.proto',
  package='i4t.protobuf',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1di4t/protobuf/Instancing.proto\x12\x0ci4t.protobuf\x1a\x19pbose/protobuf/PbWK.proto\x1a\x19pbose/protobuf/PbWS.proto\"\xda\x01\n\x15InstancingTokenClaims\x12(\n\npublic_key\x18\x01 \x01(\x0b\x32\x14.pbose.protobuf.PbWK\x12\x0e\n\x06serial\x18\x02 \x01(\t\x12\x0b\n\x03sku\x18\x03 \x01(\t\x12\x0b\n\x03upc\x18\x04 \x01(\x04\x12\x19\n\x11manufacturer_name\x18\x05 \x01(\t\x12\x1d\n\x15manufacturer_location\x18\x06 \x01(\t\x12\x18\n\x10manufacturer_lot\x18\x07 \x01(\t\x12\x19\n\x11manufacturer_line\x18\x08 \x01(\t\"t\n\x11InstancingWarrant\x12\x0f\n\x07version\x18\x01 \x01(\r\x12#\n\x05token\x18\x02 \x01(\x0b\x32\x14.pbose.protobuf.PbWS\x12)\n\x0bprivate_key\x18\x03 \x01(\x0b\x32\x14.pbose.protobuf.PbWKb\x06proto3')
  ,
  dependencies=[pbose_dot_protobuf_dot_PbWK__pb2.DESCRIPTOR,pbose_dot_protobuf_dot_PbWS__pb2.DESCRIPTOR,])




_INSTANCINGTOKENCLAIMS = _descriptor.Descriptor(
  name='InstancingTokenClaims',
  full_name='i4t.protobuf.InstancingTokenClaims',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='public_key', full_name='i4t.protobuf.InstancingTokenClaims.public_key', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serial', full_name='i4t.protobuf.InstancingTokenClaims.serial', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sku', full_name='i4t.protobuf.InstancingTokenClaims.sku', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='upc', full_name='i4t.protobuf.InstancingTokenClaims.upc', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='manufacturer_name', full_name='i4t.protobuf.InstancingTokenClaims.manufacturer_name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='manufacturer_location', full_name='i4t.protobuf.InstancingTokenClaims.manufacturer_location', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='manufacturer_lot', full_name='i4t.protobuf.InstancingTokenClaims.manufacturer_lot', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='manufacturer_line', full_name='i4t.protobuf.InstancingTokenClaims.manufacturer_line', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=102,
  serialized_end=320,
)


_INSTANCINGWARRANT = _descriptor.Descriptor(
  name='InstancingWarrant',
  full_name='i4t.protobuf.InstancingWarrant',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='i4t.protobuf.InstancingWarrant.version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='token', full_name='i4t.protobuf.InstancingWarrant.token', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='private_key', full_name='i4t.protobuf.InstancingWarrant.private_key', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=322,
  serialized_end=438,
)

_INSTANCINGTOKENCLAIMS.fields_by_name['public_key'].message_type = pbose_dot_protobuf_dot_PbWK__pb2._PBWK
_INSTANCINGWARRANT.fields_by_name['token'].message_type = pbose_dot_protobuf_dot_PbWS__pb2._PBWS
_INSTANCINGWARRANT.fields_by_name['private_key'].message_type = pbose_dot_protobuf_dot_PbWK__pb2._PBWK
DESCRIPTOR.message_types_by_name['InstancingTokenClaims'] = _INSTANCINGTOKENCLAIMS
DESCRIPTOR.message_types_by_name['InstancingWarrant'] = _INSTANCINGWARRANT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

InstancingTokenClaims = _reflection.GeneratedProtocolMessageType('InstancingTokenClaims', (_message.Message,), dict(
  DESCRIPTOR = _INSTANCINGTOKENCLAIMS,
  __module__ = 'i4t.protobuf.Instancing_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.InstancingTokenClaims)
  ))
_sym_db.RegisterMessage(InstancingTokenClaims)

InstancingWarrant = _reflection.GeneratedProtocolMessageType('InstancingWarrant', (_message.Message,), dict(
  DESCRIPTOR = _INSTANCINGWARRANT,
  __module__ = 'i4t.protobuf.Instancing_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.InstancingWarrant)
  ))
_sym_db.RegisterMessage(InstancingWarrant)


# @@protoc_insertion_point(module_scope)



## @file applet.py
# @brief I4T Applet
#
# @copyright
# Copyright 2018 I4T <https://i4t.io>
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##

import logging, os
log = logging.getLogger().getChild(__name__)

from i4t import I4TException

from i4t.client.channel import Channel

import paho.mqtt.client as mqtt

class Applet(object):

  def register_channels(applet):
    pass

  def on_connect(applet):
    pass

  def on_disconnect(applet):
    pass

  ##############################################################################
  # public
  ##############################################################################

  def __init__(self, client, ptoken):
    self._channels = {}
    self._channels_registered = {}
    self.client = client
    self.ptoken = ptoken
    self._root_channel = self.AppletRootChannel(self, '%')
    self.register_channels()

  def user(self, var, val=None):
    return self.client.user(var, val)

  def channel(self, channel_name, channel_cls=None):
    # register a channel
    if channel_cls is not None:
      self._channels.pop(channel_name, None)
      # TODO properly close the channel
      self._channels_registered[channel_name] = channel_cls
      return None
    # get the channel instance
    else:
      if channel_name in self._channels:
        return self._channels[channel_name]
      elif channel_name in self._channels_registered:
        channel_cls = self._channels_registered[channel_name]
        self._channels[channel_name] = channel_cls(self, channel_name)
        return self._channels[channel_name]
      else:
        raise I4TException("Channel: {} has not been registered. Pass channel_cls=Channel".format(channel_name))

  def connect(self):
    # connect
    host = self._ptoken.connection.host
    port = self._ptoken.connection.port
    channel = self._ptoken.connection.channel
    self.mqtt_client.connect(host, port=port, keepalive=60)
    self.applet_root = channel
    # start thread
    self.mqtt_client.loop_start()
    return self

  def disconnect(self):
    # disconnect
    self.mqtt_client.disconnect()
    self.mqtt_client.loop_stop()
    return self

  def invite(self, member, channel_name):
    self._root_channel.tx('%p/{}'.format(member), channel_name)

  def rx(self, topic, callback):
    # rx to a topic
    mqtt_topic = "{}/{}".format(self.applet_root, topic)
    def mqtt_cb(mqtt_cb_client, mqtt_cb_userdata, mqtt_cb_message):
      callback(mqtt_cb_message.payload)
    self.mqtt_client.message_callback_add(mqtt_topic, mqtt_cb)
    self.mqtt_client.subscribe(mqtt_topic)
    log.debug("Subscribed to {}".format(mqtt_topic))
    return self

  def tx(self, topic, data):
    # tx data to a topic
    mqtt_topic = "{}/{}".format(self.applet_root, topic)
    self.mqtt_client.publish(mqtt_topic, data)
    log.debug("Published to {} with data {}".format(mqtt_topic, data))
    return self

  @property
  def client(self):
    return self._client

  @client.setter
  def client(self, client):
    self._client = client

  @property
  def ptoken(self):
    return self._ptoken

  @ptoken.setter
  def ptoken(self, ptoken):
    self._ptoken = ptoken
    self._init_connection()

  ##############################################################################
  # private
  ##############################################################################

  def _init_connection(self):
    # configure MQTT client
    serial = self.client.iwarrant.token.serial
    self.mqtt_client = mqtt.Client(client_id=serial, clean_session=True)
    # TODO: switch to token authentication
    username = "nickdev"
    password = "1k89h3lmgVzRicXk"
    self.mqtt_client.username_pw_set(username, password=password)
    # setup callbacks
    self.mqtt_client.on_connect = self._on_connect
    self.mqtt_client.on_disconnect = self._on_disconnect
    return self

  def _on_connect(self, mqtt_client, userdata, flags, rc):
    log.debug("Connection returned result: {}".format(rc))
    # join the root channel
    self._root_channel.join()
    # user callback
    self.on_connect()
    return self

  def _on_disconnect(self, mqtt_client, userdata, rc):
    log.debug("Disconnection returned result: {}".format(rc))
    # user callback
    self.on_disconnect()
    return self

  def _rx_clients_serial(self, data):
    # TODO parse data for channel name and session token
    self.channel(data)._on_invite(data)
    return self

  class AppletRootChannel(Channel):
    pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: i4t/protobuf/common.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from pbose.protobuf import PbWK_pb2 as pbose_dot_protobuf_dot_PbWK__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='i4t/protobuf/common.proto',
  package='i4t.protobuf',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x19i4t/protobuf/common.proto\x12\x0ci4t.protobuf\x1a\x19pbose/protobuf/PbWK.proto\"\xd7\x01\n\x08HashType\"\xca\x01\n\x04\x45num\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03MD5\x10)\x12\x08\n\x04SHA1\x10*\x12\n\n\x06SHA224\x10+\x12\n\n\x06SHA256\x10,\x12\n\n\x06SHA384\x10-\x12\n\n\x06SHA512\x10.\x12\x0e\n\nSHA512_224\x10/\x12\x0e\n\nSHA512_256\x10\x30\x12\x0c\n\x08SHA3_224\x10\x31\x12\x0c\n\x08SHA3_256\x10\x32\x12\x0c\n\x08SHA3_384\x10\x33\x12\x0c\n\x08SHA3_512\x10\x34\x12\x0c\n\x08SHAKE128\x10\x35\x12\x0c\n\x08SHAKE256\x10\x36\"_\n\x16\x41pplicationInformation\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x37\n\nconnection\x18\x02 \x01(\x0b\x32#.i4t.protobuf.ConnectionInformation\"\x80\x01\n\x15\x43onnectionInformation\x12:\n\x04type\x18\x01 \x01(\x0e\x32,.i4t.protobuf.ConnectionInformationType.Enum\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\r\x12\x0f\n\x07\x63hannel\x18\x04 \x01(\t\"\x99\x01\n\x19\x43onnectionInformationType\"|\n\x04\x45num\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04MQTT\x10\x01\x12\t\n\x05MQTTS\x10\x02\x12\x08\n\x04HTTP\x10\x03\x12\t\n\x05HTTPS\x10\x04\x12\x08\n\x04\x43OAP\x10\x05\x12\t\n\x05\x43OAPS\x10\x06\x12\x06\n\x02WS\x10\x07\x12\x07\n\x03WSS\x10\x08\x12\n\n\x06WEBRTC\x10\t\x12\x0b\n\x07WEBRTCS\x10\n\"q\n\x0fPeerInformation\x12\x34\n\x04type\x18\x01 \x01(\x0e\x32&.i4t.protobuf.PeerInformationType.Enum\x12(\n\npublic_key\x18\x02 \x01(\x0b\x32\x14.pbose.protobuf.PbWK\"*\n\x13PeerInformationType\"\x13\n\x04\x45num\x12\x0b\n\x07UNKNOWN\x10\x00\x62\x06proto3')
  ,
  dependencies=[pbose_dot_protobuf_dot_PbWK__pb2.DESCRIPTOR,])



_HASHTYPE_ENUM = _descriptor.EnumDescriptor(
  name='Enum',
  full_name='i4t.protobuf.HashType.Enum',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MD5', index=1, number=41,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA1', index=2, number=42,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA224', index=3, number=43,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA256', index=4, number=44,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA384', index=5, number=45,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA512', index=6, number=46,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA512_224', index=7, number=47,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA512_256', index=8, number=48,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA3_224', index=9, number=49,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA3_256', index=10, number=50,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA3_384', index=11, number=51,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHA3_512', index=12, number=52,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHAKE128', index=13, number=53,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHAKE256', index=14, number=54,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=84,
  serialized_end=286,
)
_sym_db.RegisterEnumDescriptor(_HASHTYPE_ENUM)

_CONNECTIONINFORMATIONTYPE_ENUM = _descriptor.EnumDescriptor(
  name='Enum',
  full_name='i4t.protobuf.ConnectionInformationType.Enum',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MQTT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MQTTS', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HTTP', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HTTPS', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COAP', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COAPS', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WS', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WSS', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WEBRTC', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WEBRTCS', index=10, number=10,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=546,
  serialized_end=670,
)
_sym_db.RegisterEnumDescriptor(_CONNECTIONINFORMATIONTYPE_ENUM)

_PEERINFORMATIONTYPE_ENUM = _descriptor.EnumDescriptor(
  name='Enum',
  full_name='i4t.protobuf.PeerInformationType.Enum',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=84,
  serialized_end=103,
)
_sym_db.RegisterEnumDescriptor(_PEERINFORMATIONTYPE_ENUM)


_HASHTYPE = _descriptor.Descriptor(
  name='HashType',
  full_name='i4t.protobuf.HashType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _HASHTYPE_ENUM,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=286,
)


_APPLICATIONINFORMATION = _descriptor.Descriptor(
  name='ApplicationInformation',
  full_name='i4t.protobuf.ApplicationInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='i4t.protobuf.ApplicationInformation.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='connection', full_name='i4t.protobuf.ApplicationInformation.connection', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=288,
  serialized_end=383,
)


_CONNECTIONINFORMATION = _descriptor.Descriptor(
  name='ConnectionInformation',
  full_name='i4t.protobuf.ConnectionInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='i4t.protobuf.ConnectionInformation.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host', full_name='i4t.protobuf.ConnectionInformation.host', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='i4t.protobuf.ConnectionInformation.port', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channel', full_name='i4t.protobuf.ConnectionInformation.channel', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=386,
  serialized_end=514,
)


_CONNECTIONINFORMATIONTYPE = _descriptor.Descriptor(
  name='ConnectionInformationType',
  full_name='i4t.protobuf.ConnectionInformationType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CONNECTIONINFORMATIONTYPE_ENUM,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=517,
  serialized_end=670,
)


_PEERINFORMATION = _descriptor.Descriptor(
  name='PeerInformation',
  full_name='i4t.protobuf.PeerInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='i4t.protobuf.PeerInformation.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public_key', full_name='i4t.protobuf.PeerInformation.public_key', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=672,
  serialized_end=785,
)


_PEERINFORMATIONTYPE = _descriptor.Descriptor(
  name='PeerInformationType',
  full_name='i4t.protobuf.PeerInformationType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PEERINFORMATIONTYPE_ENUM,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=787,
  serialized_end=829,
)

_HASHTYPE_ENUM.containing_type = _HASHTYPE
_APPLICATIONINFORMATION.fields_by_name['connection'].message_type = _CONNECTIONINFORMATION
_CONNECTIONINFORMATION.fields_by_name['type'].enum_type = _CONNECTIONINFORMATIONTYPE_ENUM
_CONNECTIONINFORMATIONTYPE_ENUM.containing_type = _CONNECTIONINFORMATIONTYPE
_PEERINFORMATION.fields_by_name['type'].enum_type = _PEERINFORMATIONTYPE_ENUM
_PEERINFORMATION.fields_by_name['public_key'].message_type = pbose_dot_protobuf_dot_PbWK__pb2._PBWK
_PEERINFORMATIONTYPE_ENUM.containing_type = _PEERINFORMATIONTYPE
DESCRIPTOR.message_types_by_name['HashType'] = _HASHTYPE
DESCRIPTOR.message_types_by_name['ApplicationInformation'] = _APPLICATIONINFORMATION
DESCRIPTOR.message_types_by_name['ConnectionInformation'] = _CONNECTIONINFORMATION
DESCRIPTOR.message_types_by_name['ConnectionInformationType'] = _CONNECTIONINFORMATIONTYPE
DESCRIPTOR.message_types_by_name['PeerInformation'] = _PEERINFORMATION
DESCRIPTOR.message_types_by_name['PeerInformationType'] = _PEERINFORMATIONTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HashType = _reflection.GeneratedProtocolMessageType('HashType', (_message.Message,), dict(
  DESCRIPTOR = _HASHTYPE,
  __module__ = 'i4t.protobuf.common_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.HashType)
  ))
_sym_db.RegisterMessage(HashType)

ApplicationInformation = _reflection.GeneratedProtocolMessageType('ApplicationInformation', (_message.Message,), dict(
  DESCRIPTOR = _APPLICATIONINFORMATION,
  __module__ = 'i4t.protobuf.common_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.ApplicationInformation)
  ))
_sym_db.RegisterMessage(ApplicationInformation)

ConnectionInformation = _reflection.GeneratedProtocolMessageType('ConnectionInformation', (_message.Message,), dict(
  DESCRIPTOR = _CONNECTIONINFORMATION,
  __module__ = 'i4t.protobuf.common_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.ConnectionInformation)
  ))
_sym_db.RegisterMessage(ConnectionInformation)

ConnectionInformationType = _reflection.GeneratedProtocolMessageType('ConnectionInformationType', (_message.Message,), dict(
  DESCRIPTOR = _CONNECTIONINFORMATIONTYPE,
  __module__ = 'i4t.protobuf.common_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.ConnectionInformationType)
  ))
_sym_db.RegisterMessage(ConnectionInformationType)

PeerInformation = _reflection.GeneratedProtocolMessageType('PeerInformation', (_message.Message,), dict(
  DESCRIPTOR = _PEERINFORMATION,
  __module__ = 'i4t.protobuf.common_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.PeerInformation)
  ))
_sym_db.RegisterMessage(PeerInformation)

PeerInformationType = _reflection.GeneratedProtocolMessageType('PeerInformationType', (_message.Message,), dict(
  DESCRIPTOR = _PEERINFORMATIONTYPE,
  __module__ = 'i4t.protobuf.common_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.PeerInformationType)
  ))
_sym_db.RegisterMessage(PeerInformationType)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: i4t/protobuf/Session.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from pbose.protobuf import PbWK_pb2 as pbose_dot_protobuf_dot_PbWK__pb2
from pbose.protobuf import PbWS_pb2 as pbose_dot_protobuf_dot_PbWS__pb2
from i4t.protobuf import common_pb2 as i4t_dot_protobuf_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='i4t/protobuf/Session.proto',
  package='i4t.protobuf',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1ai4t/protobuf/Session.proto\x12\x0ci4t.protobuf\x1a\x19pbose/protobuf/PbWK.proto\x1a\x19pbose/protobuf/PbWS.proto\x1a\x19i4t/protobuf/common.proto\"\xad\x01\n\x12SessionTokenClaims\x12\x35\n\x07session\x18\x01 \x01(\x0b\x32$.i4t.protobuf.ApplicationInformation\x12\x30\n\tauthority\x18\x02 \x01(\x0b\x32\x1d.i4t.protobuf.PeerInformation\x12.\n\x07\x63lients\x18\x03 \x03(\x0b\x32\x1d.i4t.protobuf.PeerInformation\"t\n\x0eSessionWarrant\x12\x0f\n\x07version\x18\x01 \x01(\r\x12#\n\x05token\x18\x02 \x01(\x0b\x32\x14.pbose.protobuf.PbWS\x12,\n\x0e\x65ncryption_key\x18\x03 \x01(\x0b\x32\x14.pbose.protobuf.PbWKb\x06proto3')
  ,
  dependencies=[pbose_dot_protobuf_dot_PbWK__pb2.DESCRIPTOR,pbose_dot_protobuf_dot_PbWS__pb2.DESCRIPTOR,i4t_dot_protobuf_dot_common__pb2.DESCRIPTOR,])




_SESSIONTOKENCLAIMS = _descriptor.Descriptor(
  name='SessionTokenClaims',
  full_name='i4t.protobuf.SessionTokenClaims',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='i4t.protobuf.SessionTokenClaims.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='authority', full_name='i4t.protobuf.SessionTokenClaims.authority', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='clients', full_name='i4t.protobuf.SessionTokenClaims.clients', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=126,
  serialized_end=299,
)


_SESSIONWARRANT = _descriptor.Descriptor(
  name='SessionWarrant',
  full_name='i4t.protobuf.SessionWarrant',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='i4t.protobuf.SessionWarrant.version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='token', full_name='i4t.protobuf.SessionWarrant.token', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryption_key', full_name='i4t.protobuf.SessionWarrant.encryption_key', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=301,
  serialized_end=417,
)

_SESSIONTOKENCLAIMS.fields_by_name['session'].message_type = i4t_dot_protobuf_dot_common__pb2._APPLICATIONINFORMATION
_SESSIONTOKENCLAIMS.fields_by_name['authority'].message_type = i4t_dot_protobuf_dot_common__pb2._PEERINFORMATION
_SESSIONTOKENCLAIMS.fields_by_name['clients'].message_type = i4t_dot_protobuf_dot_common__pb2._PEERINFORMATION
_SESSIONWARRANT.fields_by_name['token'].message_type = pbose_dot_protobuf_dot_PbWS__pb2._PBWS
_SESSIONWARRANT.fields_by_name['encryption_key'].message_type = pbose_dot_protobuf_dot_PbWK__pb2._PBWK
DESCRIPTOR.message_types_by_name['SessionTokenClaims'] = _SESSIONTOKENCLAIMS
DESCRIPTOR.message_types_by_name['SessionWarrant'] = _SESSIONWARRANT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SessionTokenClaims = _reflection.GeneratedProtocolMessageType('SessionTokenClaims', (_message.Message,), dict(
  DESCRIPTOR = _SESSIONTOKENCLAIMS,
  __module__ = 'i4t.protobuf.Session_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.SessionTokenClaims)
  ))
_sym_db.RegisterMessage(SessionTokenClaims)

SessionWarrant = _reflection.GeneratedProtocolMessageType('SessionWarrant', (_message.Message,), dict(
  DESCRIPTOR = _SESSIONWARRANT,
  __module__ = 'i4t.protobuf.Session_pb2'
  # @@protoc_insertion_point(class_scope:i4t.protobuf.SessionWarrant)
  ))
_sym_db.RegisterMessage(SessionWarrant)


# @@protoc_insertion_point(module_scope)
